<?php
/**
 * Created by JetBrains PhpStorm.
 * User: paulopimenta
 * Date: 07/01/13
 * Time: 09:37
 * To change this template use File | Settings | File Templates.
 */

namespace Planejamento\Logic\Cardapio;

use \Planejamento\Entities\Cardadia;
use \Planejamento\Logic\Cardapio\Exception\RestricoesCardadia;

class Resto
{
    public static function validaCamposFiltro($filial, $servico, $mes_escolhido){
        if (empty($filial) === true)
            RestricoesCardadia::informeFilial();
        if (empty($servico) === true)
            RestricoesCardadia::informeServico();
        if (empty($mes_escolhido) === true)
            RestricoesCardadia::informeMes();
        if(self::verificaDataProcessamento(\Geral\Util\DateUtil::getDataDeString('01/'.$mes_escolhido)))
            RestricoesCardadia::mesMenorDtProcessamento();
    }

    public static function montaCamposFiltro($nrorg, $filial, $servico, $mes_escolhido){
        $dia_inicial    = \Geral\Util\DateUtil::getDataDeString('01/'.$mes_escolhido);
        $dia_final = \Geral\Util\DateUtil::getUltimoDiaDoMes($dia_inicial);

        $campos_filtro = array('NRORG' => $nrorg,
            'FILIAL' => $filial,
            'SERVICO' =>$servico,
            'DIA_INICIAL' => $dia_inicial->format(\Geral\Util\DateUtil::FORMATO_BRASILEIRO),
            'DIA_FINAL' => $dia_final->format(\Geral\Util\DateUtil::FORMATO_BRASILEIRO),
        );

        return $campos_filtro;
    }

    public static function verificaDataProcessamento($data){
        if(empty($data)) return false;

        $dataProcessamento = \Geral\Util\DateUtil::getDataDeString(\Estoque\Util\Parametros::getDataProcessamento());

        return \Geral\Util\DateUtil::comparaDatas($data, $dataProcessamento, \Geral\Util\DateUtil::DEPOIS);
    }

    public static function verificaVrPesoPorcaoRestoIngesta($campos){
        $entity = \Planejamento\Entities\Paramplaneja::findOneByCdfilialAndNrorg($campos['FILIAL'], $campos["NRORG"]);

        if($entity instanceof \Setforms2\Shared\ORM\BasicEntity){
            $vrPesoPorcRestoIngesta = (int)$entity->getVrpesoporcrein();

            if($vrPesoPorcRestoIngesta != 0){
                $where = " CDFILIAL   = ?
                            AND NRORG = ?
                            AND CDSERVFILI = ?
                            AND DTCARDAPIO BETWEEN ? AND ?
                            AND (NVL(VRPESPORPERCRI, 0) = 0)";
                $bind = array(
                    $campos['FILIAL'],
                    $campos['NRORG'],
                    $campos['SERVICO'],
                    $campos['DIA_INICIAL'],
                    $campos['DIA_FINAL'],
                );

                /** @var $entities  \Planejamento\Entities\Cardadia*/
                $entities = \Planejamento\Entities\Cardadia::getObjectsByWhere($where, $bind);
                if ($entities instanceof \Setforms2\Shared\ORM\BasicEntity) {
                    foreach($entities as $entity){
                        $entity->setVrpesporpercri($vrPesoPorcRestoIngesta);
                        $entity->persist();
                        $connection = \Setforms2\Shared\Registry::getEntityManager();
                        $connection->flush();
                    }
                }
                return true;
            }
        }
        return false;
    }

    public static function verificaVrPesoPorcaoSobraLimpa($campos){
        /** @var $entity  \Planejamento\Entities\Paramplaneja */
        $entity = \Planejamento\Entities\Paramplaneja::findOneByCdfilialAndNrorg($campos['FILIAL'], $campos["NRORG"]);

        if($entity instanceof \Setforms2\Shared\ORM\BasicEntity){
            $vrPesoPorcSobraLimpa = (int)$entity->getVrpesoporcsoli();

            if($vrPesoPorcSobraLimpa != 0){
                $where = "CDFILIAL = ?
                            AND NRORG = ?
                            AND CDSERVFILI = ?
                            AND DTCARDAPIO BETWEEN ? AND ?
                            AND (NVL(VRPESPORPERC, 0) = 0)";
                $bind = array(
                    $campos['FILIAL'],
                    $campos['NRORG'],
                    $campos['SERVICO'],
                    $campos['DIA_INICIAL'],
                    $campos['DIA_FINAL'],
                );

                /** @var $entities  \Planejamento\Entities\Cardadia*/
                $entities = \Planejamento\Entities\Cardadia::getObjectsByWhere($where, $bind);
                if ($entities instanceof \Setforms2\Shared\ORM\BasicEntity) {
                    foreach($entities as $entity){
                        $entity->setVrpesoporcsoli($vrPesoPorcSobraLimpa);
                        $entity->persist();
                        $connection = \Setforms2\Shared\Registry::getEntityManager();
                        $connection->flush();
                    }
                }
                return true;
            }
        }
        return false;
    }

    public static function calculaRestoIngesta($qt_comens_efetivos, $qt_rest_ingesta, $vr_peso_porcao){
        if ((int)$qt_comens_efetivos !== 0) {
            $array['QTGRAMPERCAP'] = (float)($qt_rest_ingesta / $qt_comens_efetivos);
        } else {
            $array['QTGRAMPERCAP'] = 0;
        }

        if ((int)$qt_comens_efetivos !== 0 & (int)$vr_peso_porcao !== 0) {
            $array['PERESTINGE'] = (float)($qt_rest_ingesta / ($qt_comens_efetivos * $vr_peso_porcao) * 100);
        } else {
            $array['PERESTINGE'] = 0;
        }

        return $array;
    }

    public static function calculaSobraLimpa($qt_comens_efetivos, $qt_sobra_limpa, $vr_peso_porcao){
        if ((int)$qt_comens_efetivos !== 0) {
            $array['QTGRAMPERCAP'] = (float)($qt_sobra_limpa / $qt_comens_efetivos);
        } else {
            $array['QTGRAMPERCAP'] = 0;
        }

        if ((int)$qt_comens_efetivos !== 0 & (int)$vr_peso_porcao !== 0) {
            $array['PESOBRLIMP'] = (float)($qt_sobra_limpa / ($qt_comens_efetivos * $vr_peso_porcao) * 100);
        } else {
            $array['PESOBRLIMP'] = 0;
        }

        return $array;
    }

    public static function qry_MediaTotal_RestoIngesta($campos){
        $array['VRPEMETAREIN'] = Cardadia::getRepository()->get_Meta_RestoIngesta($campos);
        $array['TOTALMEDIAMESDIA'] = Cardadia::getRepository()->get_TotalMediaMesDia_RestoIngesta($campos);
        $array['PEMAX'] = Cardadia::getRepository()->get_Maior_RestoIngesta($campos);
        $array['PEMIN'] = Cardadia::getRepository()->get_Menor_RestoIngesta($campos);

        return $array;
    }

    public static function qry_MediaTotal_SobraLimpa($campos){
        $array['VRPEMETASOLI'] = Cardadia::getRepository()->get_Meta_SobraLimpa($campos);
        $array['TOTALMEDIAMESDIA'] = reset(Cardadia::getRepository()->get_TotalMediaMesDia_SobraLimpa($campos));
        $array['PEMAX'] = Cardadia::getRepository()->get_Maior_SobraLimpa($campos);
        $array['PEMIN'] = Cardadia::getRepository()->get_Menor_SobraLimpa($campos);

        return $array;
    }

    public static function verificaPrato($nrorg, $cdprato) {
        if($cdprato == null)
            return true;

        $where = 'NRORG = ? AND CDPRATO = ?';
        $bind  = array($nrorg, $cdprato);

        $prato = \Planejamento\Entities\Prato::getObjectByWhere($where, $bind);

        if($prato instanceof \Planejamento\Entities\Prato)
//        if($prato instanceof \Setforms2\Shared\ORM\BasicEntity && $cdprato != 0)
            return $prato;
        else
            RestricoesCardadia::pratoNaoCadastrado();
    }
}
